function allpars=graphfit(infilename,errortype,fitborder,initdisp1,initdisp2,initfecund,initborder,initnbk,bordertype,circlerad)
% graphfit('bothupall1.txt',1,1,0.04,38.1,43.5,40.0,100,1,40)
% graphfit.m FINDS THE LIKELIHOOD (or other measure of fit) and GRAPHS THE OBSERVED VS. EXPECTED for the passed-in parameters values.
% with observed data from the file infilename
%%%%%%%%%%%%%%%%%%%%%%%%%%
% infilename should be the name of a file of a format like file 'seedbasic3.txt'
% this file should have no header line and just three columns of tab-deliminted data:
% the first column is an index saying which dataset the data in this line belongs to
% (this index should have an integer value), this feature is included is to make it easy to run lots of datasets in a row
% the second column has distances from the edge
% the third column has seed densities at those distances
% so for example, a typical set of lines might be
% 0     -20     46
% 0     -10     36
% 0     0       17
% 0     10      8
% 0     20      2
% 1     -20     35
% 1     -10     31
% 1     0       13
% 1     10      5
% 1     20      1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% errortype is a parameter that determines the type of errors to use in the fits:
% 0 = least square
% 1=Poisson
% 2=negative binomial
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% fitborder is a parameter that says whether to fit the border location or assume it is zero:
% 0 = linear edge at 0
% 1 = linear edge fitted  (i.e., fit border)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% initdisp is the initial value of the dispersal parameter
% initfecund is the initial value of the fecundity parameter
% initborder is the initial value of the border location (must be specified even if fitborder=0, although in this case it is not used)
% initnbk is the initial value of the negative binomial clumping parameter (must be specified even if errortype is not 2, although then it is not used)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% bordertype is a parameter that says what the shape of the border is and where the plants are relative to it, with the following options
% 1 = border is straight, with plants in the area with distance values less than the distance of the border, and no plants at higher distance values
% -1 = border is straight, with no plants in the area with distance values less than the distance of the border, and plants at higher distance values
% 2 = border is circular, with plants inside the circle.  The inside of the circle is at distance values smaller than the border.
% -2 = border is circular, with plants inside the circle.  The inside of the circle is at distance values larger than the border.
% 3 = border is circular, with plants outside the circle.  The inside of the circle is at distance values larger than the border.
% -3 = border is circular, with plants outside the circle.  The inside of the circle is at distance values smaller than the border.
% Note - in general, positive bordertype values mean plants are in the area with smaller distance values; negative mean plants are in the area with larger distance values.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% circlerad gives the radius of the circular area (in the same units as the distances to be read in)
% note this must be supplied even if a straight-line border case is treated (but in that case it won't be used, obviously)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% npars is the number of parameters to be fit
if (fitborder==0)
  if (errortype<2)
     useparams=[initdisp1 initdisp2 initfecund];  % initial values of dispersal and fecundity parameters
     npars=3
  else
     useparams=[initdisp1 initdisp2 initfecund initnbk];  % initial values of dispersal, fecundity, and negative binomial clumping parameters
     npars=4
  end
elseif (fitborder==1)
  if (errortype<2)
     useparams=[initdisp1 initdisp2 initfecund initborder];  % initial values of dispersal, fecundity and border parameters
     npars=4
  else
     useparams=[initdisp1 initdisp2 initfecund initborder initnbk];   % initial values of dispersal, fecundity, border and negative binomial clumping parameters
     npars=5
  end
end
[i rall sall]=textread(infilename,'%f %f %f')  % read in identifiers (which dataset), distances and densities of seeds

datasets=unique(i)
ndatasets=length(datasets)

allpars=zeros(ndatasets,npars)

for j = 1:ndatasets
  % pull out current dataset
  thisdataset=datasets(j)
  r = rall(i==thisdataset)
  s = sall(i==thisdataset)

  useloglike=totdev(useparams,r,s,errortype,fitborder,bordertype,circlerad);

  % make figure of data and fits
  npoints=200;
  mindist=min(r);
  maxdist=max(r);
  difdist=(maxdist-mindist)/npoints;
  showdists=[mindist:difdist:maxdist];
  showseeds=showdists;
  for n=1:npoints+1
    if (fitborder==0)
        showseeds(n)=sdposint(showdists(n),useparams(1),useparams(2),useparams(3),bordertype,circlerad);
    else
        showseeds(n)=sdposint(showdists(n)-useparams(4),useparams(1),useparams(2),useparams(3),bordertype,circlerad);
    end
  end
  plot(r,s,'o',showdists,showseeds,'-')
  xlabel('Distance (m)')
  ylabel('Seed density (m^-2)')
  if (fitborder==0 & errortype==0)
    plotname=sprintf('Data from %s set %i: best fit c1=%g, c2=%g, k=%g, SSQ=%g',infilename,thisdataset,useparams(1),useparams(2),useparams(3),useloglike)
  elseif (fitborder==0 & errortype==1)
    plotname=sprintf('Data from %s set %i: best fit c1=%g, c2=%g, k=%g, LL=-%g',infilename,thisdataset,useparams(1),useparams(2),useparams(3),useloglike)
  elseif (fitborder==0 & errortype==2)
    plotname=sprintf('Data from %s set %i: best fit c1=%g, c2=%g, k=%g, nbk=%g, LL=-%g',infilename,thisdataset,useparams(1),useparams(2),useparams(3),useparams(4),useloglike)
  elseif (fitborder==1 & errortype==0)
    plotname=sprintf('Data from %s set %i: best fit c1=%g, c2=%g, k=%g, d=%g, SSQ=%g',infilename,thisdataset,useparams(1),useparams(2),useparams(3),useparams(4),useloglike)
  elseif (fitborder==1 & errortype==1)
    plotname=sprintf('Data from %s set %i: best fit c1=%g, c2=%g, k=%g, d=%g, LL=-%g',infilename,thisdataset,useparams(1),useparams(2),useparams(3),useparams(4),useloglike)
  elseif (fitborder==1 & errortype==2)
    plotname=sprintf('Data from %s set %i: best fit c1=%g, c2=%g, k=%g, d=%g, nbk=%g, LL=-%g',infilename,thisdataset,useparams(1),useparams(2),useparams(3),useparams(4),useparams(5),useloglike)
  end
  title(plotname)
  maxn=length(r);

end
