function allpars=runfit(infilename,outfilestem,errortype,fitborder,initdisp1,initdisp2,initfecund,initborder,initnbk,bordertype,circlerad,getcis)
% runfit.m FINDS THE DISPERSAL PARAMETERS
% and the fecundity parameters
% and depending on settings, potentially also the border locations
% and/or thenegative binomial clumping parameters
% THAT BEST FIT THE DATA in the file infilename
% THIS IS THE VERSION FOR 2-PARAMETER DISPERSAL KERNELS
%%%%%%%%%%%%%%%%%%%%%%%%%%
% to run this in Matlab, write a command such as
% runfit6('bothupall1.txt','all1poisclark',1,1,2,50,25,123,200,1,40,0)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% infilename should be the name of a file of a format like file 'seedbasic3.txt'
% this file should have no header line and just three columns of tab-deliminted data:
% the first column is an index saying which dataset the data in this line belongs to
% (this index should have an integer value), this feature is included is to make it easy to run lots of datasets in a row
% the second column has distances from the edge
% the third column has seed densities at those distances
% so for example, a typical set of lines might be
% 0     -20     46
% 0     -10     36
% 0     0       17
% 0     10      8
% 0     20      2
% 1     -20     35
% 1     -10     31
% 1     0       13
% 1     10      5
% 1     20      1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% outfilestem is the start of the filename to be used for output files
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% errortype is a parameter that determines the type of errors to use in the fits:
% 0 = least square
% 1=Poisson
% 2=negative binomial
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% fitborder is a parameter that says whether to fit the border location or assume it is zero:
% 0 = linear edge at 0
% 1 = linear edge fitted  (i.e., fit border)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% bordertype is a parameter that says what the shape of the border is and where the plants are relative to it, with the following options
% 1 = border is straight, with plants in the area with distance values less than the distance of the border, and no plants at higher distance values
% -1 = border is straight, with no plants in the area with distance values less than the distance of the border, and plants at higher distance values
% 2 = border is circular, with plants inside the circle.  The inside of the circle is at distance values smaller than the border.
% -2 = border is circular, with plants inside the circle.  The inside of the circle is at distance values larger than the border.
% 3 = border is circular, with plants outside the circle.  The inside of the circle is at distance values larger than the border.
% -3 = border is circular, with plants outside the circle.  The inside of the circle is at distance values smaller than the border.
% Note - in general, positive bordertype values mean plants are in the area with smaller distance values; negative mean plants are in the area with larger distance values.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% circlerad gives the radius of the circular area (in the same units as the distances to be read in)
% note this must be supplied even if a straight-line border case is treated (but in that case it won't be used, obviously)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% getcis is a parameter that specifies whether to do the extra runs to get confidence intervals on the fitted parameters
% 0 = don't get confidence intervals
% 1 = get confidence intervals
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% npars is the number of parameters to be fit
if (fitborder==0)
  if (errortype<2)
     initparams=[initdisp1 initdisp2 initfecund];  % initial values of dispersal and fecundity parameters
     npars=3
  else
     initparams=[initdisp1 initdisp2 initfecund initnbk];  % initial values of dispersal, fecundity, and negative binomial clumping parameters
     npars=4
  end
elseif (fitborder==1)
  if (errortype<2)
     initparams=[initdisp1 initdisp2  initfecund initborder];  % initial values of dispersal, fecundity and border parameters
     npars=4
  else
     initparams=[initdisp1 initdisp2  initfecund initborder initnbk];   % initial values of dispersal, fecundity, border and negative binomial clumping parameters
     npars=5
  end
end
[i rall sall]=textread(infilename,'%f %f %f')  % read in identifiers (which dataset), distances and densities of seeds

datasets=unique(i)
ndatasets=length(datasets)

allpars=zeros(ndatasets,npars)

alloutfilename=sprintf('%sall.txt',outfilestem)
allfp=fopen(alloutfilename,'w');
if (getcis)
    if (fitborder==0 & errortype<2)
        fprintf(allfp,'dataset\tc1\tc2\tk\tclo\tklo\tchi\tkhi\tloglikelihood\n');
    elseif (fitborder==0 & errortype==2)
        fprintf(allfp,'dataset\tc1\tc2\tk\tnbk\tclo\tklo\tnbklo\tchi\tkhi\tnbkhi\tloglikelihood\n');
    elseif (fitborder==1 & errortype<2)
        fprintf(allfp,'dataset\tc1\tc2\tk\td\tclo\tklo\tdlo\tchi\tkhi\tdhi\tloglikelihood\n');
    else (fitborder==1 & errortype==2)
        fprintf(allfp,'dataset\tc1\tc2\tk\td\tnbk\tclo\tklo\tdlo\tnbklo\tchi\tkhi\tdhi\tnbkhi\tloglikelihood\n');
    end
else
    if (fitborder==0 & errortype<2)
        fprintf(allfp,'dataset\tc1\tc2\tk\tloglikelihood\n');
    elseif (fitborder==0 & errortype==2)
        fprintf(allfp,'dataset\tc1\tc2\tk\tnbk\tloglikelihood\n');
    elseif (fitborder==1 & errortype<2)
        fprintf(allfp,'dataset\tc1\tc2\tk\td\tloglikelihood\n');
    else (fitborder==1 & errortype==2)
        fprintf(allfp,'dataset\tc1\tc2\tk\td\tnbk\tloglikelihood\n');
    end
end

allpars(1,:)=initparams

for j = 1:ndatasets
  % pull out current dataset
  thisdataset=datasets(j)
  r = rall(i==thisdataset)
  s = sall(i==thisdataset)

  % do fit
  bestpar=dofit(initparams,r,s,errortype,fitborder,bordertype,circlerad);
  allpars(j,:)=bestpar

  bestloglike=totdev(bestpar,r,s,errortype,fitborder,bordertype,circlerad);

  % make figure of data and fits
  npoints=200;
  mindist=min(r);
  maxdist=max(r);
  difdist=(maxdist-mindist)/npoints;
  showdists=[mindist:difdist:maxdist];
  showseeds=showdists;
  for n=1:npoints+1
    if (fitborder==0)
        showseeds(n)=sdposint(showdists(n),bestpar(1),bestpar(2),bestpar(3),bordertype,circlerad);
    else
        showseeds(n)=sdposint(showdists(n)-bestpar(4),bestpar(1),bestpar(2),bestpar(3),bordertype,circlerad);
    end
  end
  plot(r,s,'s',showdists,showseeds, '-k','LineWidth',3,'MarkerEdgeColor','k','MarkerFaceColor','k','MarkerSize',6)
  xlabel('Distance (cm)','fontsize', 24, 'fontname', 'arial')
  ylabel('Seed density (m^2)','fontsize', 24, 'fontname', 'arial')
  text ('fontsize', 16, 'fontname', 'arial')
  if (fitborder==0 & errortype==0)
    plotname=sprintf('Data from %s set %i: best fit c1=%g,c2=%g, k=%g, SSQ=%g',infilename,thisdataset,bestpar(1),bestpar(2),bestpar(3),bestloglike)
elseif (fitborder==0 & errortype==1)
    plotname=sprintf('Data from %s set %i: best fit c1=%g,c2=%g, k=%g, LL=-%g',infilename,thisdataset,bestpar(1),bestpar(2),bestpar(3),bestloglike)
elseif (fitborder==0 & errortype==2)
    plotname=sprintf('Data from %s set %i: best fit c1=%g,c2=%g, k=%g, nbk=%g, LL=-%g',infilename,thisdataset,bestpar(1),bestpar(2),bestpar(3),bestpar(4),bestloglike)
elseif (fitborder==1 & errortype==0)
    plotname=sprintf('Data from %s set %i: best fit c1=%g,c2=%g, k=%g, d=%g, SSQ=%g',infilename,thisdataset,bestpar(1),bestpar(2),bestpar(3),bestpar(4),bestloglike)
elseif (fitborder==1 & errortype==1)
    plotname=sprintf('Data from %s set %i: best fit c1=%g,c2=%g, k=%g, d=%g, LL=-%g',infilename,thisdataset,bestpar(1),bestpar(2),bestpar(3),bestpar(4),bestloglike)
elseif (fitborder==1 & errortype==2)
    plotname=sprintf('Data from %s set %i: best fit c1=%g,c2=%g, k=%g, d=%g, nbk=%g, LL=-%g',infilename,thisdataset,bestpar(1),bestpar(2),bestpar(3),bestpar(4),bestpar(5),bestloglike)
end
  title(plotname)
  maxn=length(r);
 outfilename=[outfilestem,'.jpg']
 saveas(gcf,outfilename,'jpg');


  % get confidence intervals of parameter values
  if (getcis==1)
    thiscipars=dofitci(bestpar,r,s,errortype,fitborder,bordertype,circlerad);
  end

  % output fitted parameter values to a general file
  fprintf(allfp,'%i',thisdataset)
  if (getcis==1)
    nout=length(thiscipars)
    for p=1:nout
        fprintf(allfp,'\t%g',thiscipars(p))
    end
  else
    nout=length(bestpar)
    for p=1:nout
        fprintf(allfp,'\t%g',bestpar(p))
    end
  end
  fprintf(allfp,'\t%g\n',bestloglike)

  % output fitted values and parameter values to individual files (one per dataset)
  outfilename=sprintf('%s%i.txt',outfilestem,thisdataset)
  fp=fopen(outfilename,'w');
  if (fitborder==0 & errortype<2)
    fprintf(fp,'distance(cm)\tactseeddensity\tfitseeddensity\tc1\tc2\tk\n');
  elseif (fitborder==0 & errortype==2)
    fprintf(fp,'distance(cm)\tactseeddensity\tfitseeddensity\tc1\tc2\tk\tnbk\n');
  elseif (fitborder==1 & errortype<2)
    fprintf(fp,'distance(cm)\tactseeddensity\tfitseeddensity\tc1\tc2\tk\td\n');
  else (fitborder==1 & errortype==2)
    fprintf(fp,'distance(cm)\tactseeddensity\tfitseeddensity\tc1\tc2\tk\td\tnbk\n');
  end
  if (fitborder==0 & errortype<2)
    fprintf(fp,'%g\t%g\t%g\t%g\t%g\t%g\n',r(1),s(1),sdposint(r(1),bestpar(1),bestpar(2),bestpar(3),bordertype,circlerad),bestpar(1),bestpar(2),bestpar(3));
  elseif (fitborder==0 & errortype==2)
    fprintf(fp,'%g\t%g\t%g\t%g\t%g\t%g\n',r(1),s(1),sdposint(r(1),bestpar(1),bestpar(2),bestpar(3),bordertype,circlerad),bestpar(1),bestpar(2),bestpar(3),bestpar(4));
  elseif (fitborder==1 & errortype<2)
    fprintf(fp,'%g\t%g\t%g\t%g\t%g\t%g\n',r(1),s(1),sdposint(r(1)-bestpar(4),bestpar(1),bestpar(2),bestpar(3),bordertype,circlerad),bestpar(1),bestpar(2),bestpar(3),bestpar(4));
  elseif (fitborder==1 & errortype==2)
    fprintf(fp,'%g\t%g\t%g\t%g\t%g\t%g\n',r(1),s(1),sdposint(r(1)-bestpar(4),bestpar(1),bestpar(2),bestpar(3),bordertype,circlerad),bestpar(1),bestpar(2),bestpar(3),bestpar(4),bestpar(5));
  end
  for n=2:maxn
    if (fitborder==0)
      fprintf(fp,'%g\t%g\t%g\n',r(n),s(n),sdposint(r(n),bestpar(1),bestpar(2),bestpar(3),bordertype,circlerad));
    else
      fprintf(fp,'%g\t%g\t%g\n',r(n),s(n),sdposint(r(n)-bestpar(3),bestpar(1),bestpar(2),bestpar(3),bordertype,circlerad));
    end
  end
  fclose(fp);

end
fclose(allfp);
